
#include "TComUpsampleFilter.h"
#include "TypeDef.h"

#if SVC_UPSAMPLING
// ====================================================================================================================
// Tables:
// 1. PHASE_DERIVATION_IN_INTEGER = 0 is the implementation using 2 multi-phase (12 phase and 8 phase) filter sets 
//    by following K0378.
// 2. PHASE_DERIVATION_IN_INTEGER = 1 is the implemetation using a 16-phase filter set just for speed up. Some phases 
//    is approximated to x/16 (e.g. 1/3 -> 5/16). 
// 3. It was confirmed that two implementations provides the identical result. 
// 4. By default, PHASE_DERIVATION_IN_INTEGER is set to 1. 
// ====================================================================================================================
#define CNU -1 ///< Coefficients Not Used

#if TE4_K0034
#if !SYMM_SUF_COEFF
int m_fcoef_1to2_2d[4][4][49]=
{
   {
     // phase 0
     {    
    -2,    -3,    -1,     1,    -1,    -2,     0,
     5,     4,     3,     2,     3,     5,     1,
    -8,    -6,    -2,     4,   -10,    -5,    -6,
    12,     2,    25,    88,    26,     1,    12,
    -6,    -5,    -9,     5,    -3,    -6,    -8,
     2,     4,     3,     2,     3,     2,     5,
    -2,    -1,    -1,     1,    -1,    -2,    -3
     },
     // phase 1
     {                                                
     0,    -3,    -3,     1,     0,    -3,     0,
     0,     6,     4,     2,     2,     6,     2,
     0,   -10,    -7,     4,    -3,    -9,    -8,
     0,    15,     0,    67,    67,     1,    15,
     0,    -8,    -9,    -3,     4,    -7,   -10,
     0,     3,     4,     3,     3,     3,     6,
     0,    -2,    -1,     0,     1,    -2,    -3
     },
     // phase 2
     {                                                
     0,     0,     0,     0,     0,     0,     0,
    10,    -2,     3,     6,     1,     2,    -8,
   -21,     0,    -5,   -12,    -6,     0,    12,
    21,    -2,    16,    57,     8,    -2,   -15,
   -15,    -2,     7,    57,    16,    -2,    21,
    12,     0,    -6,   -10,    -6,    -1,   -20,
    -9,     3,     2,     6,     3,    -1,    10
     },
     // phase 3
     {                                               
     0,     0,     0,     0,     0,     0,     0,
     0,    11,    -2,     6,     4,     2,    -8,
     0,   -23,     0,   -10,   -10,    -2,    12,
     0,    23,    -3,    47,    37,    -5,   -15,
     0,   -14,    -6,    36,    47,    -3,    22,
     0,    12,    -2,   -10,    -9,    -1,   -21,
     0,    -9,     3,     4,     6,    -1,    10
     }
   },
   {
     // phase 0
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   128,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0
     },
     // phase 1
     {                        
     0,     3,    -1,    -2,    -7,     3,     0,
     0,    -1,    -5,    15,    11,     0,    -3,
     0,     1,     5,   -20,   -19,     2,     3,
     0,     9,   -32,   100,   105,   -34,    11,
     0,     4,     1,   -16,   -20,     4,     1,
     0,    -4,     0,    10,    14,    -4,    -1,
     0,     0,     2,    -7,    -2,    -1,     3
     },
     // phase 2
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,     0,    -7,    16,     0,     5,    -4,
    -2,     2,    11,   -32,    -4,    -6,     6,
     0,     8,   -26,   100,    -9,    16,    -9,
    -9,    16,   -11,   105,   -27,     8,     0,
     6,    -6,    -4,   -34,    10,     2,    -3,
    -5,     5,     0,    17,    -7,     0,     0
     },
     // phase 3
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,     2,    -9,     8,     7,     5,    -4,
     0,    -4,    12,   -13,   -23,    -2,     2,
     0,    10,   -25,    43,    58,   -11,     6,
     0,     6,   -11,    59,    48,   -28,    11,
     0,     2,    -2,   -23,   -16,    14,    -4,
     0,    -3,     5,     7,     9,   -10,     2
     }
   },
   {
     // phase 0
     {                        
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   128,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0
     },
     // phase 1
     {                        
     0,     0,    -2,     1,    -3,     1,    -1,
     0,     2,     1,     4,     3,     2,     0,
     0,    -1,    -4,    -5,    -7,    -2,    -2,
     0,    11,   -24,    89,    90,   -24,    12,
     0,    -1,    -3,    -4,    -6,    -4,    -2,
     0,     0,     1,     3,     4,     2,     1,
     0,    -1,     0,    -1,    -1,     0,    -1
     },
     // phase 2
     {                        
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,     1,     7,     2,     0,     1,
     1,     1,    -1,   -23,    -4,     0,    -2,
    -3,     4,    -8,    87,    -3,     5,     1,
     0,     5,    -4,    88,    -8,     3,    -3,
    -2,     0,    -4,   -23,    -1,     2,     0,
     1,     0,     1,     8,     0,     0,     0
     },
     // phase 3
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,     0,    -3,     7,     4,     1,     0,
     0,     0,     4,   -14,   -18,     3,    -4,
     0,     4,   -18,    50,    54,   -15,     8,
     0,     8,   -16,    54,    51,   -18,     4,
     0,    -3,     3,   -16,   -15,     5,    -1,
     0,     0,    -1,     6,     5,    -1,     0      
     }
   },
   {
     // phase 0
     {                              
     0,    -1,     1,     0,     1,    -1,     0,
    -1,     3,     0,    -4,    -1,     1,     1,
    -2,     0,    -3,    14,     7,    -2,    -4,
     4,    -4,    22,    65,    23,    -3,     3,
    -5,    -2,     9,    15,    -2,    -2,    -1,
     1,     2,    -1,    -4,     0,     1,    -1,
     0,    -1,     0,     0,     0,     0,     0
     },
     // phase 1
     {                          
     0,     0,    -1,     1,    -1,     0,    -1,
     0,    -1,     3,    -3,    -3,     2,     1,
     0,     0,    -5,     7,    13,     2,    -5,
     0,     2,    -1,    53,    51,     2,     1,
     0,    -5,     1,    17,     6,    -4,    -1,
     0,     2,     1,    -3,    -3,     2,    -1,
     0,    -1,    -1,     2,    -1,     1,    -1
     },
     // phase 2
     {                                        
     0,     0,     0,     0,     0,     0,     0,
     0,     1,     1,    -1,     0,    -1,     0,
    -3,     2,    -4,    -4,     0,     2,    -2,
     2,    -3,    10,    47,    20,    -3,    -1,
    -1,    -5,    21,    48,    11,    -2,     2,
    -2,     2,     0,    -4,    -4,     0,    -1,
     0,     0,    -1,     0,     1,     1,    -1
     },
     // phase 3
     {                        
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,     1,     1,    -1,     1,    -2,
     0,    -1,     0,    -5,    -3,     2,    -1,
     0,     2,    -5,    34,    40,     3,    -3,
     0,    -3,     1,    43,    33,    -1,     1,
     0,    -1,     2,    -1,    -6,    -1,    -1,
     0,     0,    -1,     1,    -1,     3,    -2
     }
   }
};
int m_fcoef_2to3_2d[4][9][49]=
{
   {
     // phase 0
     {    
    -3,    -2,    -1,     0,     0,    -1,    -2,
     5,     3,     3,     0,     2,     4,     4,
    -9,    -6,     0,     9,    -7,    -6,   -10,
    14,    -1,    31,    75,    26,     4,    13,
    -8,    -7,    -5,     6,     0,    -6,    -9,
     4,     2,     2,     1,     4,     3,     5,
    -2,    -1,     0,     0,    -2,    -1,    -3
     },
     // phase 1
     {
     0,    -4,    -2,     0,     0,    -2,    -2,
     0,     6,     4,     1,     1,     5,     4,
     0,   -12,    -4,     7,     0,    -9,   -11,
     0,    15,     8,    69,    53,     0,    16,
     0,   -10,    -9,     5,     4,    -7,   -10,
     0,     4,     4,     1,     2,     4,     5,
     0,    -2,    -1,     0,     0,    -2,    -3
     },
     // phase 2
     {
     0,    -4,    -1,    -1,     0,     0,    -3,
     0,     6,     3,     3,     0,     3,     6,
     0,   -10,    -7,     7,     2,    -7,   -14,
     0,    13,     5,    47,    67,    14,    14,
     0,    -9,    -7,    -3,     9,    -5,   -12,
     0,     4,     3,     2,     2,     4,     6,
     0,    -2,    -1,     0,    -1,    -1,    -4
     },
     // phase 3
     {
     0,     0,     0,     0,     0,     0,     0,
     8,     1,     2,     3,     1,     4,    -7,
   -17,    -6,    -2,    -7,    -6,    -4,     7,
    19,     1,    22,    65,    16,     0,    -7,
   -13,    -8,     9,    35,    12,    -3,    11,
    10,     3,    -4,    -6,    -3,     1,   -13,
    -7,     0,     2,     2,     3,    -2,     6
     },
     // phase 4
     {
     0,     0,     0,     0,     0,     0,     0,
     0,     9,     2,     2,     2,     2,    -6,
     0,   -20,    -5,    -4,    -9,    -2,     6,
     0,    19,    10,    55,    42,    -5,    -5,
     0,   -15,    -6,    31,    26,    -3,    11,
     0,     9,     1,    -7,    -3,    -1,   -12,
     0,    -7,     1,     3,     1,     1,     5
     },
     // phase 5
     {
     0,     0,     0,     0,     0,     0,     0,
     0,     9,     0,     4,     1,     5,    -7,
     0,   -21,    -2,    -5,    -7,    -6,     6,
     0,    21,     1,    44,    52,     8,    -8,
     0,   -17,    -3,    14,    36,     3,     9,
     0,    11,     0,    -4,    -6,     1,   -14,
     0,    -7,     1,     1,     4,    -2,     6
     },
     // phase 6
     {
     0,     0,     0,     0,     0,     0,     0,
     5,     0,     0,     3,     0,     2,    -8,
   -12,    -2,     0,    -7,    -2,     0,    11,
    11,    -2,    11,    38,     3,    -3,   -16,
    -5,    -6,    22,    60,    24,     1,    21,
     7,    -1,    -8,    -3,    -6,    -2,   -20,
    -6,     1,     4,     1,     4,    -1,     9
     },
     // phase 7
     {
     0,     0,     0,     0,     0,     0,     0,
     0,     5,     1,     1,     2,     0,    -7,
     0,   -13,    -1,    -3,    -7,     2,    10,
     0,    10,     3,    32,    21,    -9,   -15,
     0,    -5,     1,    56,    43,     1,    22,
     0,     5,    -4,    -7,    -2,    -6,   -20,
     0,    -6,     2,     4,     1,     2,     9
     },
     // phase 8
     {
     0,     0,     0,     0,     0,     0,     0,
     0,     6,    -1,     2,     1,     3,    -8,
     0,   -14,     1,    -4,    -4,    -2,    11,
     0,    12,    -2,    25,    25,     0,   -18,
     0,    -8,     1,    33,    60,     9,    22,
     0,     8,    -6,    -5,    -6,    -2,   -23,
     0,    -7,     4,     1,     4,     0,    10
     }
   },
   {
     // phase 0
     {
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   128,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0
     },
     // phase 1
     {                        
     0,     3,    -3,    -1,    -7,     2,     0,
     0,    -3,     1,    11,     7,     0,    -2,
     0,     5,    -5,   -13,   -13,     1,     3,
     0,     4,   -23,   125,    63,   -16,     2,
     0,     7,    -6,   -10,   -15,     2,     2,
     0,    -4,     2,     9,    10,    -1,    -2,
     0,     1,    -1,    -5,    -3,    -1,     2
     },
     // phase 2
     {                        
     0,     2,     0,    -2,    -6,    -1,     1,
     0,    -2,    -2,     9,    10,     2,    -3,
     0,     2,     2,   -13,   -12,    -6,     5,
     0,     2,   -17,    61,   126,   -21,     4,
     0,     4,     1,   -11,   -13,    -6,     5,
     0,    -3,     0,     6,    11,     2,    -4,
     0,     0,     2,    -6,    -1,    -4,     4
     },
     // phase 3
     {                        
     0,     0,     0,     0,     0,     0,     0,
    -1,    -1,    -4,    13,     1,     4,    -4,
    -1,     6,     1,   -26,    -9,    -2,     4,
    -2,     3,   -14,   122,    -2,    11,    -8,
    -4,    10,    -5,    59,   -17,     3,     1,
     2,    -4,    -4,   -16,     6,     4,    -3,
    -2,     4,    -1,     9,    -6,     1,     0
     },
     // phase 4
     {                        
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,    -7,    11,     3,     4,    -3,
     0,     2,     9,   -22,   -16,    -4,     4,
     0,     6,   -28,    92,    47,    -5,     0,
     0,     7,    -8,    46,    11,   -12,     5,
     0,    -1,    -3,   -15,    -3,     8,    -3,
     0,     0,     2,     5,     1,    -5,     1
     },
     // phase 5
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,     0,    -6,     5,     7,     4,    -2,
     0,     0,    10,   -11,   -23,    -6,     3,
     0,     4,   -19,    39,    95,   -12,     5,
     0,     2,     0,    20,    41,   -21,     6,
     0,     1,    -4,    -9,   -12,    10,    -2,
     0,    -2,     4,     1,     7,    -8,     1
     },
     // phase 6
     {                        
     0,     0,     0,     0,     0,     0,     0,
     0,     0,    -5,     7,    -1,     4,    -2,
    -2,     2,     5,   -13,    -4,    -4,     2,
     1,     4,   -15,    54,    -5,    11,    -4,
    -7,    10,    -3,   127,   -13,     2,    -2,
     4,    -1,    -9,   -28,    -1,     7,    -1,
    -4,     4,     0,    15,    -4,    -1,    -2
     },
     // phase 7
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,     1,    -7,     5,     0,     4,    -2,
     0,    -1,     8,    -8,    -8,    -4,     1,
     0,     6,   -19,    37,    18,     1,     2,
     0,     6,   -15,    99,    42,   -19,     4,
     0,     3,    -5,   -25,   -14,    10,     0,
     0,    -2,     3,     9,     5,    -6,    -1
     },
     // phase 8
     {                        
     0,     0,     0,     0,     0,     0,     0,
     0,     1,    -4,     1,     3,     2,     0,
     0,    -2,     6,    -1,   -12,    -3,    -1,
     0,     4,   -10,     9,    41,    -6,     6,
     0,     1,    -8,    49,    96,   -27,     5,
     0,     3,    -2,   -17,   -24,     8,     2,
     0,    -3,     3,     4,    12,    -7,    -1
     }
   },
   {
     // phase 0
     {
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   128,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0
     },
     // phase 1
     {                      
     0,     0,    -2,     1,    -3,     1,    -1,
     0,     1,     3,     1,     1,     2,     0,
     0,    -1,    -8,     3,    -5,    -1,    -2,
     0,     8,   -16,   106,    58,   -14,     6,
     0,     0,    -6,     2,    -5,    -3,    -2,
     0,     0,     2,     1,     2,     2,     1,
     0,    -1,    -1,    -1,    -1,     0,     0
     },
     // phase 2
     {                          
     0,     0,    -1,     0,    -2,     0,    -1,
     0,     1,     2,     3,    -1,     3,     0,
     0,    -1,    -4,    -3,     1,    -6,     0,
     0,     6,   -13,    56,   105,   -15,     7,
     0,    -1,    -1,    -4,     2,    -7,    -1,
     0,     0,     1,     2,     1,     2,     1,
     0,    -1,     0,    -1,    -1,     0,    -1
     },
     // phase 3
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,     1,     4,     1,     0,     1,
     0,     2,    -5,   -16,    -6,     1,    -2,
    -1,     0,     1,   105,     3,     2,     1,
    -1,     4,    -3,    58,    -6,     0,    -3,
    -1,     0,    -1,   -15,     0,     2,     0,
     0,     0,     0,     4,     0,    -1,     0
     },
     // phase 4
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,    -1,     5,     1,     1,     0,
     0,     1,     0,   -14,   -13,     3,    -3,
     0,     4,   -17,    84,    46,   -11,     5,
     0,     4,   -10,    46,    19,    -9,     0,
     0,    -2,     2,   -13,    -7,     3,     0,
     0,     0,     0,     4,     2,    -1,     0
     },
     // phase 5
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,     0,    -1,     4,     3,     2,    -1,
     0,     0,     1,    -9,   -17,     0,    -2,
     0,     3,   -13,    42,    85,   -13,     7,
     0,     2,    -5,    21,    44,   -13,     1,
     0,    -2,     1,    -7,   -12,     3,     1,
     0,     0,     0,     2,     3,    -1,    -1
     },
     // phase 6
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,     1,     3,     1,     0,     0,
     1,     2,    -1,   -14,    -2,     0,    -1,
    -3,     1,    -5,    56,    -3,     3,    -1,
     1,     2,     3,   105,     1,     0,    -1,
    -2,     1,    -7,   -15,    -6,     2,    -1,
     1,     0,     2,     4,     1,     0,     0
     },
     // phase 7
     {                              
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,    -2,     4,     0,     1,     0,
     0,     1,     2,   -10,    -8,     2,    -2,
     0,     1,   -13,    44,    20,    -5,     2,
     0,     7,   -14,    85,    44,   -14,     3,
     0,    -2,    -1,   -14,   -11,     2,     0,
     0,    -1,     1,     4,     3,     0,     0
     },
     // phase 8
     {                        
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,    -1,     3,     2,     1,    -1,
     0,     0,     2,    -6,   -12,     2,    -2,
     0,     1,    -9,    20,    42,    -8,     3,
     0,     5,   -10,    43,    85,   -16,     4,
     0,    -2,     1,   -11,   -13,    -2,     1,
     0,     0,     1,     3,     3,     1,    -1
     }
   },
   {
     // phase 0
     {
    -1,     3,    -3,    -1,     0,     1,    -2,
    -2,     3,     2,    -1,    -1,     2,     1,
     0,    -3,    -3,    12,     8,     0,    -5,
     0,    -3,    28,    59,    25,     1,    -1,
    -5,     1,     9,    10,    -5,    -2,     1,
     0,     3,    -2,     2,     2,     2,    -1,
    -2,     1,    -1,    -1,    -3,     1,    -1
     },
     // phase 1
     {                          
     0,    -1,    -1,     0,    -3,     2,    -3,
     0,     0,     2,     1,    -3,     3,     1,
     0,    -1,    -5,     9,    11,     3,    -5,
     0,    -2,    11,    57,    43,     3,    -2,
     0,    -4,     5,    12,     0,    -4,     0,
     0,     1,     0,     1,     1,     3,    -1,
     0,    -2,     0,     1,    -5,     1,    -1
     },
     // phase 2
     {                            
     0,    -1,     1,    -1,    -2,     2,    -3,
     0,    -1,     3,     1,    -3,     2,     1,
     0,     0,    -5,     3,    12,     7,    -6,
     0,    -2,     4,    42,    53,    17,    -5,
     0,    -5,     4,    11,     3,    -3,    -1,
     0,     0,     2,     0,     1,     4,    -1,
     0,    -2,     0,     0,    -3,     0,    -1
     },
     // phase 3
     {                      
     0,     0,     0,     0,     0,     0,     0,
    -2,     3,    -2,    -1,    -1,     1,    -3,
     0,     2,    -3,    -1,     1,     2,    -1,
     1,    -5,    18,    51,    25,     1,    -3,
    -4,     0,    21,    33,     5,    -3,     0,
    -1,     3,    -3,    -4,    -1,     3,    -1,
    -2,     1,    -1,     1,    -2,     1,    -1
     },
     // phase 4
     {                        
     0,     0,     0,     0,     0,     0,     0,
     0,    -2,     0,     2,    -3,     3,    -3,
     0,     2,    -3,    -1,    -1,     3,    -2,
     0,    -3,     4,    45,    40,     4,    -3,
     0,    -5,    12,    35,    16,    -4,     0,
     0,     1,     0,    -4,    -4,     3,     0,
     0,    -3,     0,     2,    -3,     2,    -2
     },
     // phase 5
     {                        
     0,     0,     0,     0,     0,     0,     0,
     0,    -2,     1,     0,    -2,     2,    -4,
     0,     1,     0,    -3,    -1,     4,    -1,
     0,    -1,    -1,    32,    46,    18,    -6,
     0,    -6,     6,    29,    24,     2,    -3,
     0,     1,     1,    -4,    -5,     2,     1,
     0,    -3,     1,     2,    -2,     1,    -2
     },
     // phase 6
     {                                  
     0,     0,     0,     0,     0,     0,     0,
    -2,     3,    -2,    -1,     0,     1,    -2,
    -2,     3,    -1,    -2,    -2,     2,    -1,
     2,    -5,     6,    30,    19,     3,    -5,
    -2,    -1,    29,    50,    13,    -3,     1,
    -3,     1,     1,     2,    -2,     2,     0,
    -2,     3,    -2,    -1,    -1,     1,    -2
     },
     // phase 7
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,    -1,     1,    -3,     4,    -3,
     0,     0,     1,    -1,    -4,     1,     0,
     0,    -1,    -4,    26,    27,     6,    -4,
     0,    -4,    14,    50,    30,    -2,    -1,
     0,    -2,     2,     3,    -2,     1,     0,
     0,    -2,     0,     1,    -3,     2,    -3
     },
     // phase 8
     {                          
     0,     0,     0,     0,     0,     0,     0,
     0,    -2,     1,    -1,    -2,     2,    -3,
     0,    -1,     1,     0,    -3,     1,    -1,
     0,     0,    -3,    13,    31,    16,    -6,
     0,    -4,     7,    41,    40,     7,    -4,
     0,    -3,     2,     3,    -1,     0,     1,
     0,    -2,     1,     0,    -2,     1,    -2
     }
   }
};
#else
// symmetric filters
int m_fcoef_1to2_2d[4][4][49]=
{
	{
    //phase_0	
    {		   		  		 		
    -3,     1,    -4,     2,    -3,     0,    -1,
     5,     2,     6,     1,     6,     2,     2,
    -6,     0,   -13,     5,   -15,     0,    -5,
     6,     2,    11,   127,    11,     2,     6,
    -5,     0,   -15,     5,   -13,     0,    -6,
     2,     2,     6,     1,     6,     1,     5,
    -1,     0,    -3,     2,    -4,     1,    -3
     },
    //phase_1	
    {		   		  		 		
     0,    -2,    -1,    -1,    -1,    -2,    -1,
     0,     4,     5,     3,     3,     6,     1,
     0,    -4,   -10,    -2,    -5,    -9,    -3,
     0,    12,   -14,    85,    85,   -14,    12,
     0,    -3,    -9,    -5,    -2,   -10,    -4,
     0,     1,     6,     3,     3,     5,     4,
     0,    -1,    -2,    -1,    -1,    -1,    -2
     },
    //phase_2	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     3,     1,     2,     7,     1,     1,    -2,
    -8,     1,    -8,   -17,    -6,     1,     3,
     8,     1,     1,    82,    -3,     2,    -6,
    -6,     2,    -3,    82,     1,     1,     8,
     3,     1,    -6,   -17,    -8,     1,    -8,
    -2,     1,     1,     7,     2,     1,     3
     },
    //phase_3	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,     4,     0,     6,     5,     1,    -2,
     0,    -8,     0,   -15,   -14,     1,     2,
     0,    12,   -14,    53,    48,   -13,    -2,
     0,    -2,   -13,    48,    53,   -14,    12,
     0,     2,     1,   -14,   -15,     0,    -8,
     0,    -2,     1,     5,     6,     0,     4
     }
	},
	{
    //phase_0	
    {								
     1,     0,     0,     0,     1,    -2,     0,
    -1,     1,     0,    -5,    -1,     1,     1,
    -1,    -1,     0,    15,     7,    -4,    -4,
     3,    -4,    25,    65,    25,    -4,     3,
    -4,    -4,     7,    15,     0,    -1,    -1,
     1,     1,    -1,    -5,     0,     1,    -1,
     0,    -2,     1,     0,     0,    -1,     1
     },
    //phase_1	
    {								
     0,     0,     0,     0,     1,     0,    -1,
     0,    -1,     2,    -3,    -4,     0,     1,
     0,    -1,    -3,     8,    14,     0,    -5,
     0,     1,     2,    53,    53,     2,     1,
     0,    -5,     0,    14,     8,    -3,    -1,
     0,     1,     0,    -4,    -3,     2,    -1,
     0,    -1,     0,     1,     0,     0,     0
     },
    //phase_2	
    {								
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     1,    -1,     0,    -1,     0,
    -2,     0,    -4,    -4,    -1,     0,    -1,
     2,    -2,    14,    48,    21,    -5,    -1,
    -1,    -5,    21,    48,    14,    -2,     2,
    -1,     0,    -1,    -4,    -4,     0,    -2,
     0,    -1,     0,    -1,     1,     0,     0
     },
    //phase_3		
    {							
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     1,     0,     0,     0,     0,
     0,    -2,    -1,    -5,    -3,     0,    -1,
     0,     1,    -1,    35,    42,     2,    -4,
     0,    -4,     2,    42,    35,    -1,     1,
     0,    -1,     0,    -3,    -5,    -1,    -2,
     0,     0,     0,     0,     0,     1,     0
     }
	},
	{
    //phase0	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   128,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0
     },
    //phase1	
    {		   		  		 		
     0,     2,    -1,    -2,    -7,     3,     0,
     0,    -2,    -3,    13,    10,     0,    -3,
     0,     1,     4,   -20,   -17,     2,     3,
     0,    11,   -34,   104,   104,   -34,    11,
     0,     3,     2,   -17,   -20,     4,     1,
     0,    -3,     0,    10,    13,    -3,    -2,
     0,     0,     3,    -7,    -2,    -1,     2
     },
    //phase2		
    {	   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,    -5,    15,     1,     3,    -2,
    -2,     3,     8,   -32,    -5,    -4,     4,
    -1,     7,   -25,   102,    -9,    14,    -7,
    -7,    14,    -9,   102,   -25,     7,    -1,
     4,    -4,    -5,   -32,     8,     3,    -2,
    -2,     3,     1,    15,    -5,    -1,     0
     },
    //phase3	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,     1,    -8,     8,     7,     4,    -3,
     0,    -4,    13,   -15,   -23,     0,     0,
     0,     9,   -26,    47,    60,   -13,     7,
     0,     7,   -13,    60,    47,   -26,     9,
     0,     0,     0,   -23,   -15,    13,    -4,
     0,    -3,     4,     7,     8,    -8,     1
     }
	},
	{
    //phase_0	
    {		   		  		 		
     0,     1,    -3,     2,     0,     0,    -1,
     0,     0,     5,     1,    -2,     1,     1,
    -3,     5,   -16,     8,    -9,     3,    -2,
     3,    -4,    11,   126,    11,    -4,     3,
    -2,     3,    -9,     8,   -16,     5,    -3,
     1,     1,    -2,     1,     5,     0,     0,
    -1,     0,     0,     2,    -3,     1,     0
     },
    //phase_1		
    {	   		  		 		
     0,     0,    -2,     0,     0,     0,    -1,
     0,     0,     3,     3,    -1,     2,     0,
     0,     2,    -8,    -2,     0,    -7,     2,
     0,     6,   -16,    83,    83,   -16,     6,
     0,     2,    -7,     0,    -2,    -8,     2,
     0,     0,     2,    -1,     3,     3,     0,
     0,    -1,     0,     0,     0,    -2,     0
     },
    //phase_2	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
    -1,    -1,     0,     7,     2,     0,     0,
     0,     5,    -6,   -15,   -12,     4,    -2,
    -1,    -1,    -3,    81,     5,     0,     2,
     2,     0,     5,    81,    -3,    -1,    -1,
    -2,     4,   -12,   -15,    -6,     5,     0,
     0,     0,     2,     7,     0,    -1,    -1
     },
    //phase_3	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,    -2,     5,     5,     1,     0,
     0,     0,     4,   -12,   -18,    -2,     0,
     0,     4,   -17,    44,    59,   -11,     5,
     0,     5,   -11,    59,    44,   -17,     4,
     0,     0,    -2,   -18,   -12,     4,     0,
     0,     0,     1,     5,     5,    -2,    -1
     }
	}
};
int m_fcoef_2to3_2d[4][9][49]=
{	
	{
    //phase_0
    {
    -2,     1,    -4,     0,    -3,     0,    -1,
     4,     2,     6,     0,     4,     3,     2,
    -5,    -3,   -11,     9,   -12,    -2,    -5,
     6,    -1,    19,   113,    19,    -1,     6,
    -5,    -2,   -12,     9,   -11,    -3,    -5,
     2,     3,     4,     0,     6,     2,     4,
    -1,     0,    -3,     1,    -4,     1,    -2
     },
    //phase_1	
    {		   		  		 		
     0,    -2,    -2,    -1,    -2,    -1,    -1,
     0,     4,     5,     2,     2,     5,     2,
     0,    -4,   -12,     6,    -7,    -6,    -5,
     0,     8,    -3,    96,    60,    -8,     8,
     0,    -5,   -11,     3,    -2,    -9,    -4,
     0,     3,     6,     0,     4,     4,     4,
     0,    -1,    -3,     0,    -3,     0,    -2
     },
    //phase_2	
    {		   		  		 		
     0,    -2,     0,    -3,     0,    -3,    -1,
     0,     4,     4,     4,     0,     6,     3,
     0,    -4,    -9,    -2,     3,   -11,    -5,
     0,     8,    -8,    60,    96,    -3,     8,
     0,    -5,    -6,    -7,     6,   -12,    -4,
     0,     2,     5,     2,     2,     5,     4,
     0,    -1,    -1,    -2,    -1,    -2,    -2
     },
    //phase_3	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     3,     1,     2,     4,     2,     2,    -2,
    -7,     0,    -9,   -10,   -10,     0,     1,
     7,    -2,    12,    93,    11,    -1,    -3,
    -6,    -2,    -2,    52,     0,    -3,     3,
     4,     2,    -3,   -10,    -3,     2,    -4,
    -3,     1,     0,     4,     0,     1,     1
     },
    //phase_4	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,     3,     2,     4,     2,     3,    -2,
     0,    -6,    -6,   -10,   -12,    -2,     1,
     0,     8,    -6,    78,    45,    -8,     0,
     0,    -5,    -9,    40,    22,   -10,     5,
     0,     4,     1,   -11,    -5,     1,    -3,
     0,    -2,     0,     3,     1,     1,     1
     },
    //phase_5		 
    {	   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,     3,     2,     3,     3,     3,    -2,
     0,    -7,    -2,   -10,   -12,    -5,     1,
     0,     9,   -10,    47,    76,    -5,     0,
     0,    -5,    -7,    19,    44,   -11,     4,
     0,     4,     2,    -8,    -9,     1,    -3,
     0,    -3,     1,     1,     3,    -1,     2
     },
    //phase_6	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     1,     1,     0,     4,     0,     1,    -3,
    -4,     2,    -3,   -10,    -3,     2,     4,
     3,    -3,     0,    52,    -2,    -2,    -6,
    -3,    -1,    11,    93,    12,    -2,     7,
     1,     0,   -10,   -10,    -9,     0,    -7,
    -2,     2,     2,     4,     2,     1,     3
     },
    //phase_7	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,     2,    -1,     3,     1,     1,    -3,
     0,    -3,     1,    -9,    -8,     2,     4,
     0,     4,   -11,    44,    19,    -7,    -5,
     0,     0,    -5,    76,    47,   -10,     9,
     0,     1,    -5,   -12,   -10,    -2,    -7,
     0,    -2,     3,     3,     3,     2,     3
     },
    //phase_8	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,     1,     1,     1,     3,     0,    -2,
     0,    -3,     1,    -5,   -11,     1,     4,
     0,     5,   -10,    22,    40,    -9,    -5,
     0,     0,    -8,    45,    78,    -6,     8,
     0,     1,    -2,   -12,   -10,    -6,    -6,
     0,    -2,     3,     2,     4,     2,     3
     }
	},
	{
    //phase_0
    {
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   128,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0
     },
    //phase_1
    {									
     0,     3,    -3,    -3,    -7,     2,     1,
     0,    -3,     0,    14,     8,    -1,    -3,
     0,     4,    -3,   -18,   -13,     2,     3,
     0,     5,   -25,   132,    65,   -18,     2,
     0,     6,    -5,   -14,   -18,     5,     1,
     0,    -4,     2,    11,    12,    -4,    -1,
     0,     1,    -1,    -6,    -3,     0,     2
     },
    //phase_2	
    {								
     0,     2,     0,    -3,    -6,    -1,     1,
     0,    -1,    -4,    12,    11,     2,    -4,
     0,     1,     5,   -18,   -14,    -5,     6,
     0,     2,   -18,    65,   132,   -25,     5,
     0,     3,     2,   -13,   -18,    -3,     4,
     0,    -3,    -1,     8,    14,     0,    -3,
     0,     1,     2,    -7,    -3,    -3,     3
     },
    //phase_3	
    {								
     0,     0,     0,     0,     0,     0,     0,
    -1,    -2,    -4,    13,     2,     3,    -3,
    -2,     7,     2,   -30,    -9,    -2,     4,
    -1,     3,   -16,   129,    -3,    11,    -8,
    -5,    12,    -5,    56,   -19,     5,     1,
     3,    -5,    -3,   -15,     8,     2,    -3,
    -2,     4,     0,     7,    -6,     0,     0
     },
    //phase_4
    {								
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,    -8,    12,     4,     4,    -3,
     0,     1,    11,   -25,   -18,    -3,     4,
     0,     6,   -30,    95,    51,    -8,     1,
     0,     6,    -6,    45,     7,    -9,     4,
     0,    -1,    -4,   -14,     0,     7,    -3,
     0,     0,     3,     3,     0,    -5,     2
     },
    //phase_5
    {								
     0,     0,     0,     0,     0,     0,     0,
     0,     0,    -7,     5,     8,     4,    -2,
     0,    -1,    11,   -12,   -26,    -6,     3,
     0,     4,   -19,    39,   102,   -14,     5,
     0,     2,     0,    21,    36,   -20,     6,
     0,     1,    -4,    -9,   -10,    10,    -2,
     0,    -2,     4,     1,     6,    -8,     2
     },
    //phase_6
    {								
     0,     0,     0,     0,     0,     0,     0,
     0,     0,    -6,     7,     0,     4,    -2,
    -3,     2,     8,   -15,    -3,    -5,     3,
     1,     5,   -19,    56,    -5,    12,    -5,
    -8,    11,    -3,   129,   -16,     3,    -1,
     4,    -2,    -9,   -30,     2,     7,    -2,
    -3,     3,     2,    13,    -4,    -2,    -1
     },
    //phase_7	
    {								
     0,     0,     0,     0,     0,     0,     0,
     0,     2,    -8,     6,     1,     4,    -2,
     0,    -2,    10,   -10,    -9,    -4,     1,
     0,     6,   -20,    36,    21,     0,     2,
     0,     5,   -14,   102,    39,   -19,     4,
     0,     3,    -6,   -26,   -12,    11,    -1,
     0,    -2,     4,     8,     5,    -7,     0
     },
    //phase_8	
    {								
     0,     0,     0,     0,     0,     0,     0,
     0,     2,    -5,     0,     3,     3,     0,
     0,    -3,     7,     0,   -14,    -4,    -1,
     0,     4,    -9,     7,    45,    -6,     6,
     0,     1,    -8,    51,    95,   -30,     6,
     0,     4,    -3,   -18,   -25,    11,     1,
     0,    -3,     4,     4,    12,    -8,    -1
     }
	},
	{
    //phase_0
    {
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   128,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0
     },
    //phase1	
    {		   		  		 		
     0,    -1,    -1,    -1,    -2,     0,    -1,
     0,     1,     3,     1,     3,     2,     0,
     0,     0,    -8,     1,    -7,    -1,    -2,
     0,     9,   -20,   113,    58,   -15,     7,
     0,     0,    -7,     0,    -5,    -3,    -1,
     0,     0,     3,     1,     2,     2,     1,
     0,    -1,     0,    -2,     0,    -1,     0
     },
    //phase2		
    {	   		  		 		
     0,     0,    -1,     0,    -2,     0,    -1,
     0,     1,     2,     2,     1,     3,     0,
     0,    -1,    -3,    -5,     0,    -7,     0,
     0,     7,   -15,    58,   113,   -20,     9,
     0,    -2,    -1,    -7,     1,    -8,     0,
     0,     0,     2,     3,     1,     3,     1,
     0,    -1,     0,    -2,    -1,    -1,    -1
     },
    //phase3	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,     2,     5,     2,    -1,     1,
    -1,     3,    -7,   -17,    -7,     3,    -3,
    -2,     2,    -2,   114,    -2,     4,     0,
    -1,     4,    -8,    61,    -8,     3,    -3,
    -1,     1,    -2,   -15,    -2,     2,     0,
     0,     0,     1,     4,     0,    -1,     0
     },
    //phase4		   
    {   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,     0,     5,     3,     0,     0,
     0,     1,    -1,   -16,   -13,     2,    -3,
     0,     6,   -19,    91,    44,   -12,     6,
     0,     4,   -13,    46,    19,    -9,     1,
     0,    -2,     3,   -14,    -7,     4,     0,
     0,    -1,     0,     3,     2,    -1,     0
     },
    //phase5		 
    {	   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,    -1,     4,     4,     1,    -1,
     0,     0,     2,   -12,   -18,     0,    -2,
     0,     4,   -14,    45,    90,   -18,     8,
     0,     3,    -7,    18,    47,   -15,     3,
     0,    -2,     3,    -8,   -13,     3,     1,
     0,     0,     0,     2,     4,    -1,    -1
     },
    //phase6		  
    {   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,     0,     4,     1,     0,     0,
     0,     2,    -2,   -15,    -2,     1,    -1,
    -3,     3,    -8,    61,    -8,     4,    -1,
     0,     4,    -2,   114,    -2,     2,    -2,
    -3,     3,    -7,   -17,    -7,     3,    -1,
     1,    -1,     2,     5,     2,    -1,     0
     },
    //phase7		   
    {   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,    -1,     4,     2,     0,     0,
     0,     1,     3,   -13,    -8,     3,    -2,
     0,     3,   -15,    47,    18,    -7,     3,
     0,     8,   -18,    90,    45,   -14,     4,
     0,    -2,     0,   -18,   -12,     2,     0,
     0,    -1,     1,     4,     4,    -1,    -1
     },
    //phase8		
    {	   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,     0,    -1,     2,     3,     0,    -1,
     0,     0,     4,    -7,   -14,     3,    -2,
     0,     1,    -9,    19,    46,   -13,     4,
     0,     6,   -12,    44,    91,   -19,     6,
     0,    -3,     2,   -13,   -16,    -1,     1,
     0,     0,     0,     3,     5,     0,    -1
     }
	},
	{
    //phase_0
    {
     0,     0,    -2,     0,     0,     0,    -2,
    -2,     3,     0,    -1,    -3,     1,     1,
     0,    -3,    -1,    14,     8,    -2,    -5,
     0,     1,    28,    57,    28,     1,     0,
    -5,    -2,     8,    14,    -1,    -3,     0,
     1,     2,    -3,    -1,     0,     3,    -2,
    -2,     0,     0,     0,    -2,     0,     0
     },
    //phase_1		
    {	   		  		 		
     0,    -1,    -1,     0,    -1,     1,    -2,
     0,     0,     2,    -1,    -3,     1,     1,
     0,    -3,    -2,     9,    11,     2,    -6,
     0,    -3,    17,    52,    42,     6,    -2,
     0,    -6,     4,    14,     5,    -4,    -1,
     0,     1,     0,    -2,    -1,     3,    -1,
     0,    -2,     0,     1,    -2,     1,    -1
     },
    //phase_2	
    {	   		  		 		
     0,    -1,     1,    -2,     1,     0,    -2,
     0,    -1,     3,    -1,    -2,     0,     1,
     0,    -1,    -4,     5,    14,     4,    -6,
     0,    -2,     6,    42,    52,    17,    -3,
     0,    -6,     2,    11,     9,    -2,    -3,
     0,     1,     1,    -3,    -1,     2,     0,
     0,    -2,     1,    -1,     0,    -1,    -1
     },
    //phase_3	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
    -1,     1,    -1,    -1,    -2,     1,    -1,
    -1,     0,    -2,     3,     0,     0,    -2,
     0,    -2,    19,    49,    26,     0,    -2,
    -4,     0,    18,    33,     9,    -3,     0,
     0,     0,    -3,    -3,    -2,     2,    -1,
    -1,     1,    -1,     0,    -1,     1,    -1
     },
    //phase_4		   
    {   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,    -2,     1,     0,    -3,     2,    -2,
     0,     0,    -2,     1,     2,     0,    -3,
     0,    -3,    10,    43,    38,     6,    -4,
     0,    -6,    12,    32,    19,    -1,    -1,
     0,    -1,    -1,    -3,    -3,     1,    -1,
     0,    -2,     1,     0,    -2,     1,    -1
     },
    //phase_5		  
    {   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,    -2,     2,    -2,    -1,     0,    -2,
     0,    -1,    -1,    -2,     2,     2,    -3,
     0,    -1,     2,    32,    46,    16,    -4,
     0,    -5,     4,    27,    28,     3,    -3,
     0,     0,     0,    -4,    -3,     0,     0,
     0,    -2,     2,    -1,     0,     0,    -1
     },
    //phase_6		
    {	   		  		 		
     0,     0,     0,     0,     0,     0,     0,
    -1,     1,    -1,     0,    -1,     1,    -1,
    -1,     2,    -2,    -3,    -3,     0,     0,
     0,    -3,     9,    33,    18,     0,    -4,
    -2,     0,    26,    49,    19,    -2,     0,
    -2,     0,     0,     3,    -2,     0,    -1,
    -1,     1,    -2,    -1,    -1,     1,    -1
     },
    //phase_7	
    {		   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,     0,     0,    -1,     2,    -2,
     0,     0,     0,    -3,    -4,     0,     0,
     0,    -3,     3,    28,    27,     4,    -5,
     0,    -4,    16,    46,    32,     2,    -1,
     0,    -3,     2,     2,    -2,    -1,    -1,
     0,    -2,     0,    -1,    -2,     2,    -2
     },
    //phase_8		
    {	   		  		 		
     0,     0,     0,     0,     0,     0,     0,
     0,    -1,     1,    -2,     0,     1,    -2,
     0,    -1,     1,    -3,    -3,    -1,    -1,
     0,    -1,    -1,    19,    32,    12,    -6,
     0,    -4,     6,    38,    43,    10,    -3,
     0,    -3,     0,     2,     1,    -2,     0,
     0,    -2,     2,    -3,     0,     1,    -2
     }
	}
};
#endif
#endif

#if PHASE_DERIVATION_IN_INTEGER
const Int TComUpsampleFilter::m_lumaFixedFilter[16][NTAPS_US_LUMA] =
{
  {  0,  0,  0, 64,  0,  0,  0,  0}, //
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}, //
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}, //
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}, // 
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}, //
  { -1, 4, -11, 52, 26,  -8, 3, -1}, // <-> actual phase shift 1/3, used for spatial scalability x1.5      
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}, //       
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}, // 
  { -1, 4, -11, 40, 40, -11, 4, -1}, // <-> actual phase shift 1/2, equal to HEVC MC, used for spatial scalability x2
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}, // 
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}, // 
  { -1, 3,  -8, 26, 52, -11, 4, -1}, // <-> actual phase shift 2/3, used for spatial scalability x1.5
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}, // 
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}, // 
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}, // 
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU}  // 
};

const Int TComUpsampleFilter::m_chromaFixedFilter[16][NTAPS_US_CHROMA] =
{
  {  0, 64,  0,  0},//
  {CNU,CNU,CNU,CNU},//
  {CNU,CNU,CNU,CNU},//
  {CNU,CNU,CNU,CNU},// 
  { -4, 54, 16, -2},// <-> actual phase shift 1/4,equal to HEVC MC, used for spatial scalability x1.5 (only for accurate Chroma alignement)
  { -5, 50, 22, -3},// <-> actual phase shift 1/3, used for spatial scalability x1.5   
  { -6, 46, 28, -4},// <-> actual phase shift 3/8,equal to HEVC MC, used for spatial scalability x2 (only for accurate Chroma alignement)      
  {CNU,CNU,CNU,CNU},// 
  { -4, 36, 36, -4},// <-> actual phase shift 1/2,equal to HEVC MC, used for spatial scalability x2
  { -4, 30, 43, -5},// <-> actual phase shift 7/12, used for spatial scalability x1.5 (only for accurate Chroma alignement)
  {CNU,CNU,CNU,CNU},// 
  { -3, 22, 50, -5},// <-> actual phase shift 2/3, used for spatial scalability x1.5
  {CNU,CNU,CNU,CNU},// 
  {CNU,CNU,CNU,CNU},// 
  { -2, 10, 58, -2},// <-> actual phase shift 7/8,equal to HEVC MC, used for spatial scalability x2 (only for accurate Chroma alignement)  
  { -1,  5, 62, -2} // <-> actual phase shift 11/12, used for spatial scalability x1.5 (only for accurate Chroma alignement)
};
#else
const Int TComUpsampleFilter::m_lumaFixedFilter[12][NTAPS_US_LUMA] =
{
  { 0,  0,   0, 64,  0,   0, 0,  0},
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU},//1/12 
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU},//2/12 
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU},//3/12 
  { -1, 4, -11, 52, 26,  -8, 3, -1},//4/12 
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU},//5/12 
  { -1, 4, -11, 40, 40, -11, 4, -1},//6/12       
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU},//7/12 
  { -1, 3,  -8, 26, 52, -11, 4, -1},//8/12 
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU},//9/12 
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU},//10/12
  {CNU,CNU,CNU,CNU,CNU,CNU,CNU,CNU},//11/12
};

const Int TComUpsampleFilter::m_chromaFixedFilter15[12][NTAPS_US_CHROMA] =
{
  {  0, 64,  0,  0},
  {CNU,CNU,CNU,CNU},//1/12 
  {CNU,CNU,CNU,CNU},//2/12 
  { -4, 54, 16, -2},//3/12 
  { -5, 50, 22, -3},//4/12 
  {CNU,CNU,CNU,CNU},//5/12 
  {CNU,CNU,CNU,CNU},//6/12 
  { -4, 30, 43, -5},//7/12 
  { -3, 22, 50, -5},//8/12 
  {CNU,CNU,CNU,CNU},//9/12 
  {CNU,CNU,CNU,CNU},//10/12
  { -1,  5, 62, -2} //11/12
};

const Int TComUpsampleFilter::m_chromaFixedFilter20[8][NTAPS_US_CHROMA] =
{
  {  0, 64,  0,  0},
  {CNU,CNU,CNU,CNU},//1/8 
  {CNU,CNU,CNU,CNU},//2/8 
  { -6, 46, 28, -4},//3/8 
  { -4, 36, 36, -4},//4/8 
  {CNU,CNU,CNU,CNU},//5/8 
  {CNU,CNU,CNU,CNU},//6/8 
  { -2, 10, 58, -2},//7/8 
};
#endif

TComUpsampleFilter::TComUpsampleFilter(void)
{
}

TComUpsampleFilter::~TComUpsampleFilter(void)
{
}
#if TE4_K0034
Void TComUpsampleFilter::FilterImgNonSep(
	 Pel           *src,             
     Pel           *dst,             
     int           height1,          
     int           width1,        
     int           height2,     
     int           width2,          
     int           OrgStride,        
     int           UpStride,         
     int           filterIdx)
{
  int phase,phase1;
  int i3,j3,i2,j2,i1,j1,i,j,i_phase,j_phase;
  int M,N;
  int iSum;
  int *filter=NULL;

  if(width1*2==width2 && height1*2==height2)
  {
    phase=4;
    phase1=2;
    M=2;N=1;
  }
  else if(width1*3==width2*2 && height1*3==height2*2)
  {
    phase=9;
    phase1=3;
    M=3;N=2;
  }
  else
  {
    assert(!"Illegal up sample ratio");
  }
  int filter_size=7;

  int offset=filter_size/2;
  j3=-N;
  for(j2=0;j2<height2;j2++)
  {
    j3+=N;  j1=j3/M;  j_phase=j3%M;
    i3=-N;
    for(i2=0;i2< width2;i2++)
    {
      i3+=N;  i1=i3/M;  i_phase=i3%M;
      i=j_phase*phase1+i_phase;
      filter=(phase==9) ? m_fcoef_2to3_2d[filterIdx][i]:m_fcoef_1to2_2d[filterIdx][i]; 

      iSum=0;
      for(j=0;j<filter_size;j++)
        for(i=0;i<filter_size;i++)
          iSum += src[(j1-offset+j)*OrgStride+(i1-offset+i)]*filter[j*filter_size+i];
      dst[j2*UpStride+i2]= Clip3(0,255,(iSum+(1<<6))>>7);
    }
  }
}
Void TComUpsampleFilter::upsampleBasePicSUF( TComPicYuv* pcUsPic, TComPicYuv* pcBasePic, TComPicYuv* pcTempPic, Int SUFIdx )
{
  Int i, j;

  //========== Y component upsampling ===========
  Int iBWidth   = pcBasePic->getWidth () - pcBasePic->getPicCropLeftOffset() - pcBasePic->getPicCropRightOffset();
  Int iBHeight  = pcBasePic->getHeight() - pcBasePic->getPicCropTopOffset() - pcBasePic->getPicCropBottomOffset();
  Int iBStride  = pcBasePic->getStride();

  Int iEWidth   = pcUsPic->getWidth () - pcUsPic->getPicCropLeftOffset() - pcUsPic->getPicCropRightOffset();
  Int iEHeight  = pcUsPic->getHeight() - pcUsPic->getPicCropTopOffset() - pcUsPic->getPicCropBottomOffset();
  Int iEStride  = pcUsPic->getStride();

#if PHASE_DERIVATION_IN_INTEGER
  Int iRefPos16 = 0;
  Int phase    = 0;
  Int refPos   = 0; 
  Int* coeff = m_chromaFilter[phase];
  for ( i = 0; i < 16; i++)
  {
    memcpy(   m_lumaFilter[i],   m_lumaFixedFilter[i], sizeof(Int) * NTAPS_US_LUMA   );
    memcpy( m_chromaFilter[i], m_chromaFixedFilter[i], sizeof(Int) * NTAPS_US_CHROMA );
  }
#else
  for ( i = 0; i < 12; i++)
  {
    memcpy( m_lumaFilter[i], m_lumaFixedFilter[i], sizeof(Int) * NTAPS_US_LUMA );
  }

  Int chromaPhaseDenominator;
  if (iEWidth == 2*iBWidth) // 2x scalability
  {
    for ( i = 0; i < 8; i++)
    {
      memcpy( m_chromaFilter[i], m_chromaFixedFilter20[i], sizeof(Int) * NTAPS_US_CHROMA );
    }
    chromaPhaseDenominator = 8;
  }
  else
  {
    for ( i = 0; i < 12; i++) // 1.5x scalability
    {
      memcpy( m_chromaFilter[i], m_chromaFixedFilter15[i], sizeof(Int) * NTAPS_US_CHROMA );
    }
    chromaPhaseDenominator = 12;
  }
#endif

  //Pel* piTempBufY = pcTempPic->getLumaAddr();
  Pel* piSrcBufY  = pcBasePic->getLumaAddr();
  Pel* piDstBufY  = pcUsPic->getLumaAddr();

  //Pel* piSrcY;
  //Pel* piDstY;

  assert ( iEWidth == 2*iBWidth || 2*iEWidth == 3*iBWidth );
  assert ( iEHeight == 2*iBHeight || 2*iEHeight == 3*iBHeight );

  pcBasePic->setBorderExtension(false);
  pcBasePic->extendPicBorder   (); // extend the border.

  FilterImgNonSep(
	  piSrcBufY, 
	  piDstBufY,                  
	  iBHeight,      
	  iBWidth,       
	  iEHeight,      
	  iEWidth,       
	  iBStride,
	  iEStride,
	  SUFIdx);

  //========== UV component upsampling ===========
  Pel* piTempBufU = pcTempPic->getCbAddr();
  Pel* piSrcBufU  = pcBasePic->getCbAddr();
  Pel* piDstBufU  = pcUsPic->getCbAddr();

  Pel* piTempBufV = pcTempPic->getCrAddr();
  Pel* piSrcBufV  = pcBasePic->getCrAddr();
  Pel* piDstBufV  = pcUsPic->getCrAddr();

  Pel* piSrcU;
  Pel* piDstU;
  Pel* piSrcV;
  Pel* piDstV;

  iEWidth  >>= 1;
  iEHeight >>= 1;

  iBWidth  >>= 1;
  iBHeight >>= 1;

  iBStride  = pcBasePic->getCStride();
  iEStride  = pcUsPic->getCStride();

#if PHASE_DERIVATION_IN_INTEGER
  Int   iShiftX = 16;
  Int   iShiftY = 16;

  Int   iPhaseX = 0;
  Int   iPhaseY = 1;

  Int   iAddX       = ( ( ( iBWidth * iPhaseX ) << ( iShiftX - 2 ) ) + ( iEWidth >> 1 ) ) / iEWidth + ( 1 << ( iShiftX - 5 ) );
  Int   iAddY       = ( ( ( iBHeight * iPhaseY ) << ( iShiftY - 2 ) ) + ( iEHeight >> 1 ) ) / iEHeight+ ( 1 << ( iShiftY - 5 ) );

  Int   iDeltaX     = 4 * iPhaseX;
  Int   iDeltaY     = 4 * iPhaseY;

  Int   iShiftXM4 = iShiftX - 4;
  Int   iShiftYM4 = iShiftY - 4;

  Int   iScaleX     = ( ( iBWidth << iShiftX ) + ( iEWidth >> 1 ) ) / iEWidth;
  Int   iScaleY     = ( ( iBHeight << iShiftY ) + ( iEHeight >> 1 ) ) / iEHeight;
#else
  const Double sFactor = 1.0 * iBWidth / iEWidth;
  const Double sFactor12 = sFactor * 12;
#endif

  //========== horizontal upsampling ===========
  for( i = 0; i < iEWidth; i++ )
  {
#if PHASE_DERIVATION_IN_INTEGER
    iRefPos16 = ((i*iScaleX + iAddX) >> iShiftXM4) - iDeltaX;
    phase    = iRefPos16 & 15;
    refPos   = iRefPos16 >> 4;
    coeff = m_chromaFilter[phase];
#else
    Int refPosM = (Int) ( i * chromaPhaseDenominator * sFactor );
    Int refPos = (Int)( i * sFactor );
    Int phase = (refPosM + chromaPhaseDenominator) % chromaPhaseDenominator;
    Int* coeff = m_chromaFilter[phase];
#endif

    piSrcU = piSrcBufU + refPos -((NTAPS_US_CHROMA>>1) - 1);
    piSrcV = piSrcBufV + refPos -((NTAPS_US_CHROMA>>1) - 1);
    piDstU = piTempBufU + i;
    piDstV = piTempBufV + i;

    for( j = 0; j < iBHeight ; j++ )
    {
      *piDstU = sumChromaHor(piSrcU, coeff);
      *piDstV = sumChromaHor(piSrcV, coeff);

      piSrcU += iBStride;
      piSrcV += iBStride;
      piDstU += iEStride;
      piDstV += iEStride;
    }
  }

  //========== vertical upsampling ===========
  pcTempPic->setBorderExtension(false);
  pcTempPic->setHeight(iBHeight << 1);
  pcTempPic->extendPicBorder   (); // extend the border.
  pcTempPic->setHeight(iEHeight << 1);

  const Int nShift = US_FILTER_PREC*2;
  Int iOffset = 1 << (nShift - 1); 

  for( j = 0; j < iEHeight; j++ )
  {
#if PHASE_DERIVATION_IN_INTEGER
    iRefPos16 = ((j*iScaleY + iAddY) >> iShiftYM4) - iDeltaY;
    phase    = iRefPos16 & 15;
    refPos   = iRefPos16 >> 4; 
    coeff = m_chromaFilter[phase];
#else
    Int refPosM = (Int) (j * chromaPhaseDenominator * sFactor) - 1;
    Int refPos; 
    if ( refPosM < 0 )
    {
      refPos = (Int)( j * sFactor ) - 1;
    }
    else
    {
      refPos = refPosM / chromaPhaseDenominator;
    }
    Int phase = (refPosM + chromaPhaseDenominator) % chromaPhaseDenominator;
    Int* coeff = m_chromaFilter[phase];
#endif

    piSrcU = piTempBufU  + (refPos -((NTAPS_US_CHROMA>>1) - 1))*iEStride;
    piSrcV = piTempBufV  + (refPos -((NTAPS_US_CHROMA>>1) - 1))*iEStride;

    piDstU = piDstBufU + j*iEStride;
    piDstV = piDstBufV + j*iEStride;

    for( i = 0; i < iEWidth; i++ )
    {
      *piDstU = Clip( (sumChromaVer(piSrcU, coeff, iEStride) + iOffset) >> (nShift));
      *piDstV = Clip( (sumChromaVer(piSrcV, coeff, iEStride) + iOffset) >> (nShift));

      piSrcU++;
      piSrcV++;
      piDstU++;
      piDstV++;
    }
  }

  pcUsPic->setBorderExtension(false);
  pcUsPic->extendPicBorder   (); // extend the border.

  //Reset the Border extension flag
  pcUsPic->setBorderExtension(false);
  pcTempPic->setBorderExtension(false);
  pcBasePic->setBorderExtension(false);
}
#endif
Void TComUpsampleFilter::upsampleBasePic( TComPicYuv* pcUsPic, TComPicYuv* pcBasePic, TComPicYuv* pcTempPic )
{
  assert ( NTAPS_US_LUMA == 8 );
  assert ( NTAPS_US_CHROMA == 4 );

  Int i, j;

  //========== Y component upsampling ===========
  Int iBWidth   = pcBasePic->getWidth () - pcBasePic->getPicCropLeftOffset() - pcBasePic->getPicCropRightOffset();
  Int iBHeight  = pcBasePic->getHeight() - pcBasePic->getPicCropTopOffset() - pcBasePic->getPicCropBottomOffset();
  Int iBStride  = pcBasePic->getStride();

  Int iEWidth   = pcUsPic->getWidth () - pcUsPic->getPicCropLeftOffset() - pcUsPic->getPicCropRightOffset();
  Int iEHeight  = pcUsPic->getHeight() - pcUsPic->getPicCropTopOffset() - pcUsPic->getPicCropBottomOffset();
  Int iEStride  = pcUsPic->getStride();

#if PHASE_DERIVATION_IN_INTEGER
  Int iRefPos16 = 0;
  Int phase    = 0;
  Int refPos   = 0; 
  Int* coeff = m_chromaFilter[phase];
  for ( i = 0; i < 16; i++)
  {
    memcpy(   m_lumaFilter[i],   m_lumaFixedFilter[i], sizeof(Int) * NTAPS_US_LUMA   );
    memcpy( m_chromaFilter[i], m_chromaFixedFilter[i], sizeof(Int) * NTAPS_US_CHROMA );
  }
#else
  for ( i = 0; i < 12; i++)
  {
    memcpy( m_lumaFilter[i], m_lumaFixedFilter[i], sizeof(Int) * NTAPS_US_LUMA );
  }

  Int chromaPhaseDenominator;
  if (iEWidth == 2*iBWidth) // 2x scalability
  {
    for ( i = 0; i < 8; i++)
    {
      memcpy( m_chromaFilter[i], m_chromaFixedFilter20[i], sizeof(Int) * NTAPS_US_CHROMA );
    }
    chromaPhaseDenominator = 8;
  }
  else
  {
    for ( i = 0; i < 12; i++) // 1.5x scalability
    {
      memcpy( m_chromaFilter[i], m_chromaFixedFilter15[i], sizeof(Int) * NTAPS_US_CHROMA );
    }
    chromaPhaseDenominator = 12;
  }
#endif

  Pel* piTempBufY = pcTempPic->getLumaAddr();
  Pel* piSrcBufY  = pcBasePic->getLumaAddr();
  Pel* piDstBufY  = pcUsPic->getLumaAddr();

  Pel* piSrcY;
  Pel* piDstY;

  assert ( iEWidth == 2*iBWidth || 2*iEWidth == 3*iBWidth );
  assert ( iEHeight == 2*iBHeight || 2*iEHeight == 3*iBHeight );

  pcBasePic->setBorderExtension(false);
  pcBasePic->extendPicBorder   (); // extend the border.

#if PHASE_DERIVATION_IN_INTEGER
  Int   iShiftX = 16;
  Int   iShiftY = 16;

  Int   iPhaseX = 0;
  Int   iPhaseY = 0;

  Int   iAddX       = ( ( ( iBWidth * iPhaseX ) << ( iShiftX - 2 ) ) + ( iEWidth >> 1 ) ) / iEWidth + ( 1 << ( iShiftX - 5 ) );
  Int   iAddY       = ( ( ( iBHeight * iPhaseY ) << ( iShiftY - 2 ) ) + ( iEHeight >> 1 ) ) / iEHeight+ ( 1 << ( iShiftY - 5 ) );

  Int   iDeltaX     = 4 * iPhaseX;
  Int   iDeltaY     = 4 * iPhaseY;  


  Int iShiftXM4 = iShiftX - 4;
  Int iShiftYM4 = iShiftY - 4;

  Int   iScaleX     = ( ( iBWidth << iShiftX ) + ( iEWidth >> 1 ) ) / iEWidth;
  Int   iScaleY     = ( ( iBHeight << iShiftY ) + ( iEHeight >> 1 ) ) / iEHeight;
#else
  const Double sFactor = 1.0 * iBWidth / iEWidth;
  const Double sFactor12 = sFactor * 12;
#endif

  //========== horizontal upsampling ===========
  for( i = 0; i < iEWidth; i++ )
  {
#if PHASE_DERIVATION_IN_INTEGER
    iRefPos16 = ((i*iScaleX + iAddX) >> iShiftXM4) - iDeltaX;
    phase    = iRefPos16 & 15;
    refPos   = iRefPos16 >> 4;
    coeff = m_lumaFilter[phase];
#else
    Int refPos12 = (Int) ( i * sFactor12 );
    Int refPos = (Int)( i * sFactor );
    Int phase = (refPos12 + 12) % 12; 
    Int* coeff = m_lumaFilter[phase];
#endif

    piSrcY = piSrcBufY + refPos -((NTAPS_US_LUMA>>1) - 1);
    piDstY = piTempBufY + i;

    for( j = 0; j < iBHeight ; j++ )
    {
      *piDstY = sumLumaHor(piSrcY, coeff);
      piSrcY += iBStride;
      piDstY += iEStride;
    }
  }


  //========== vertical upsampling ===========
  pcTempPic->setBorderExtension(false);
  pcTempPic->setHeight(iBHeight);
  pcTempPic->extendPicBorder   (); // extend the border.
  pcTempPic->setHeight(iEHeight);

  const Int nShift = US_FILTER_PREC*2;
  Int iOffset = 1 << (nShift - 1); 

  for( j = 0; j < iEHeight; j++ )
  {
#if PHASE_DERIVATION_IN_INTEGER
    iRefPos16 = ((j*iScaleY + iAddY) >> iShiftYM4) - iDeltaY;
    phase    = iRefPos16 & 15;
    refPos   = iRefPos16 >> 4;
    coeff = m_lumaFilter[phase];
#else
    Int refPos12 = (Int) (j * sFactor12 );
    Int refPos = (Int)( j * sFactor );
    Int phase = (refPos12 + 12) % 12;
    Int* coeff = m_lumaFilter[phase];
#endif

    piSrcY = piTempBufY + (refPos -((NTAPS_US_LUMA>>1) - 1))*iEStride;
    piDstY = piDstBufY + j * iEStride;

    for( i = 0; i < iEWidth; i++ )
    {
      *piDstY = Clip( (sumLumaVer(piSrcY, coeff, iEStride) + iOffset) >> (nShift));
      piSrcY++;
      piDstY++;
    }
  }

  //========== UV component upsampling ===========
  Pel* piTempBufU = pcTempPic->getCbAddr();
  Pel* piSrcBufU  = pcBasePic->getCbAddr();
  Pel* piDstBufU  = pcUsPic->getCbAddr();

  Pel* piTempBufV = pcTempPic->getCrAddr();
  Pel* piSrcBufV  = pcBasePic->getCrAddr();
  Pel* piDstBufV  = pcUsPic->getCrAddr();

  Pel* piSrcU;
  Pel* piDstU;
  Pel* piSrcV;
  Pel* piDstV;

  iEWidth  >>= 1;
  iEHeight >>= 1;

  iBWidth  >>= 1;
  iBHeight >>= 1;

  iBStride  = pcBasePic->getCStride();
  iEStride  = pcUsPic->getCStride();

#if PHASE_DERIVATION_IN_INTEGER
  iShiftX = 16;
  iShiftY = 16;

  iPhaseX = 0;
  iPhaseY = 1;

  iAddX       = ( ( ( iBWidth * iPhaseX ) << ( iShiftX - 2 ) ) + ( iEWidth >> 1 ) ) / iEWidth + ( 1 << ( iShiftX - 5 ) );
  iAddY       = ( ( ( iBHeight * iPhaseY ) << ( iShiftY - 2 ) ) + ( iEHeight >> 1 ) ) / iEHeight+ ( 1 << ( iShiftY - 5 ) );

  iDeltaX     = 4 * iPhaseX;
  iDeltaY     = 4 * iPhaseY;

  iShiftXM4 = iShiftX - 4;
  iShiftYM4 = iShiftY - 4;

  iScaleX     = ( ( iBWidth << iShiftX ) + ( iEWidth >> 1 ) ) / iEWidth;
  iScaleY     = ( ( iBHeight << iShiftY ) + ( iEHeight >> 1 ) ) / iEHeight;
#endif

  //========== horizontal upsampling ===========
  for( i = 0; i < iEWidth; i++ )
  {
#if PHASE_DERIVATION_IN_INTEGER
    iRefPos16 = ((i*iScaleX + iAddX) >> iShiftXM4) - iDeltaX;
    phase    = iRefPos16 & 15;
    refPos   = iRefPos16 >> 4;
    coeff = m_chromaFilter[phase];
#else
    Int refPosM = (Int) ( i * chromaPhaseDenominator * sFactor );
    Int refPos = (Int)( i * sFactor );
    Int phase = (refPosM + chromaPhaseDenominator) % chromaPhaseDenominator;
    Int* coeff = m_chromaFilter[phase];
#endif

    piSrcU = piSrcBufU + refPos -((NTAPS_US_CHROMA>>1) - 1);
    piSrcV = piSrcBufV + refPos -((NTAPS_US_CHROMA>>1) - 1);
    piDstU = piTempBufU + i;
    piDstV = piTempBufV + i;

    for( j = 0; j < iBHeight ; j++ )
    {
      *piDstU = sumChromaHor(piSrcU, coeff);
      *piDstV = sumChromaHor(piSrcV, coeff);

      piSrcU += iBStride;
      piSrcV += iBStride;
      piDstU += iEStride;
      piDstV += iEStride;
    }
  }

  //========== vertical upsampling ===========
  pcTempPic->setBorderExtension(false);
  pcTempPic->setHeight(iBHeight << 1);
  pcTempPic->extendPicBorder   (); // extend the border.
  pcTempPic->setHeight(iEHeight << 1);

  for( j = 0; j < iEHeight; j++ )
  {
#if PHASE_DERIVATION_IN_INTEGER
    iRefPos16 = ((j*iScaleY + iAddY) >> iShiftYM4) - iDeltaY;
    phase    = iRefPos16 & 15;
    refPos   = iRefPos16 >> 4; 
    coeff = m_chromaFilter[phase];
#else
    Int refPosM = (Int) (j * chromaPhaseDenominator * sFactor) - 1;
    Int refPos; 
    if ( refPosM < 0 )
    {
      refPos = (Int)( j * sFactor ) - 1;
    }
    else
    {
      refPos = refPosM / chromaPhaseDenominator;
    }
    Int phase = (refPosM + chromaPhaseDenominator) % chromaPhaseDenominator;
    Int* coeff = m_chromaFilter[phase];
#endif

    piSrcU = piTempBufU  + (refPos -((NTAPS_US_CHROMA>>1) - 1))*iEStride;
    piSrcV = piTempBufV  + (refPos -((NTAPS_US_CHROMA>>1) - 1))*iEStride;

    piDstU = piDstBufU + j*iEStride;
    piDstV = piDstBufV + j*iEStride;

    for( i = 0; i < iEWidth; i++ )
    {
      *piDstU = Clip( (sumChromaVer(piSrcU, coeff, iEStride) + iOffset) >> (nShift));
      *piDstV = Clip( (sumChromaVer(piSrcV, coeff, iEStride) + iOffset) >> (nShift));

      piSrcU++;
      piSrcV++;
      piDstU++;
      piDstV++;
    }
  }

  pcUsPic->setBorderExtension(false);
  pcUsPic->extendPicBorder   (); // extend the border.

  //Reset the Border extension flag
  pcUsPic->setBorderExtension(false);
  pcTempPic->setBorderExtension(false);
  pcBasePic->setBorderExtension(false);
}
#endif //SVC_EXTENSION
